package com.persagy.dmp.rwd.edit.config.web;

import com.persagy.common.json.JacksonMapper;
import com.persagy.dmp.rwd.edit.config.shiro.DmpAccessControlFilter;
import org.springframework.amqp.support.converter.Jackson2JsonMessageConverter;
import org.springframework.amqp.support.converter.MessageConverter;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.http.HttpMessageConverters;
import org.springframework.boot.web.servlet.FilterRegistrationBean;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.http.converter.json.MappingJackson2HttpMessageConverter;
import org.springframework.web.servlet.config.annotation.InterceptorRegistry;
import org.springframework.web.servlet.config.annotation.WebMvcConfigurer;

/**
 * @author: yaoll
 * @date: 2020-12-29
 * @verison: 1.0
 */
@Configuration
public class WebConfig implements WebMvcConfigurer {

    @Autowired
    private UserUtils userUtils;

    /**
     * 登录拦截器
     */
    @Override
    public void addInterceptors(InterceptorRegistry registry) { // 设置拦截的路径、不拦截的路径、优先级等等
        registry.addInterceptor(userUtils).addPathPatterns("/**");
    }

    @Bean
    public HttpMessageConverters customConverters() {
        MappingJackson2HttpMessageConverter converter = new MappingJackson2HttpMessageConverter();
        converter.setObjectMapper(JacksonMapper.nonEmptyMapper);
        return new HttpMessageConverters(converter);
    }

    @Bean
    public FilterRegistrationBean registration(DmpAccessControlFilter filter) {
        FilterRegistrationBean registration = new FilterRegistrationBean(filter);
        registration.setEnabled(false);
        return registration;
    }

    @Bean
    public MessageConverter customMessageConvert() {
        return new Jackson2JsonMessageConverter();
    }

}
