package com.persagy.dmp.rwd.edit.service;

import com.fasterxml.jackson.databind.node.ObjectNode;
import com.google.gson.Gson;
import com.google.gson.JsonSyntaxException;
import com.google.gson.reflect.TypeToken;
import com.persagy.common.criteria.CriteriaUtils;
import com.persagy.common.criteria.JacksonCriteria;
import com.persagy.common.web.MapResponse;
import com.persagy.common.web.PagedResponse;
import com.persagy.dmp.rwd.edit.config.web.UserUtils;
import com.persagy.dmp.rwd.edit.entity.FuncidDefChangeRecord;
import com.persagy.dmp.rwd.edit.entity.QChangeRecord;
import com.persagy.dmp.rwd.edit.entity.QFuncidDefChangeRecord;
import com.persagy.dmp.rwd.edit.enumeration.EnumOperationType;
import com.persagy.dmp.rwd.edit.repository.FuncidDefChangeRecordRepository;
import com.persagy.dmp.rwd.model.FuncidDefModel;
import com.querydsl.core.types.dsl.BooleanExpression;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletResponse;
import javax.transaction.Transactional;
import java.io.*;
import java.util.*;

@Slf4j
@Service
public class FuncidDefChangeRecordService {

    @Autowired
    FuncidDefChangeRecordRepository repository;

    @Autowired
    FuncidDefService funcidDefService;

    @Autowired
    CriteriaUtils criteriaUtils;

    private List<BooleanExpression> parse(ObjectNode object) {
        List<BooleanExpression> list = new LinkedList<>();
        QChangeRecord qt = QChangeRecord.changeRecord;
        list.addAll(CriteriaUtils.parse(qt.id, object.get("id")));
        list.addAll(CriteriaUtils.parse(qt.code, object.get("code")));
        list.addAll(CriteriaUtils.parse(qt.operationType, object.get("operationType"), EnumOperationType.class));
        list.addAll(CriteriaUtils.parse(qt.operationUser, object.get("operationUser")));
        list.addAll(CriteriaUtils.parse(qt.operationTime, object.get("operationTime")));
        list.addAll(CriteriaUtils.parse(qt.confirmUser, object.get("confirmUser")));
        list.addAll(CriteriaUtils.parse(qt.confirmTime, object.get("confirmTime")));
        list.addAll(CriteriaUtils.parse(qt.publishUser, object.get("publishUser")));
        list.addAll(CriteriaUtils.parse(qt.publishTime, object.get("publishTime")));
        list.addAll(CriteriaUtils.parse(qt.finishTime, object.get("finishTime")));
        list.addAll(CriteriaUtils.parse(qt.state, object.get("state")));
        return list;
    }

    public PagedResponse<FuncidDefChangeRecord> query(JacksonCriteria criteria) {
        PagedResponse<FuncidDefChangeRecord> result = criteriaUtils.query(QFuncidDefChangeRecord.funcidDefChangeRecord, this::parse, criteria);
        return result;
    }

    @Transactional
    public MapResponse create(FuncidDefChangeRecord param) {
        MapResponse response = new MapResponse();
        String code = param.getCode();
        String classCode = param.getClassCode();
        if (code == null) {
            response.setFail("code is required");
            return response;
        }
        if (classCode == null) {
            response.setFail("classCode is required");
            return response;
        }
        if (EnumOperationType.create.equals(param.getOperationType())) {
            Boolean isExists = isExists(code, classCode);
            if (!isExists) {
                response.setFail("code is exists");
                return response;
            }
        }
        QChangeRecord qt = QChangeRecord.changeRecord;
        BooleanExpression ex = qt.code.eq(code).and(qt.state.eq(1));
        Iterable<FuncidDefChangeRecord> all = repository.findAll(ex);
        List<FuncidDefChangeRecord> list = new ArrayList<>();
        all.forEach(list::add);
        if (list.size() > 0) {
            repository.deleteAll(list);
        }
        param.setOperationUser(UserUtils.currentUserId() + "");
        param.setOperationTime(new Date());
        param.setValid(true);
        param.setState(1);
        repository.save(param);
        response.add("id", param.getId());
        return response;
    }

    private Boolean isExists(String code, String classCode) {
        JacksonCriteria criteria = JacksonCriteria.newInstance();
        criteria.add("type", "common");
        criteria.add("code", code);
        criteria.add("classCode", classCode);
        PagedResponse<FuncidDefModel> query = funcidDefService.query(criteria);
        List<FuncidDefModel> data = query.getData();
        if (data.size() > 0) {
            return false;
        }
        return true;
    }

    @Transactional
    public MapResponse update(FuncidDefChangeRecord param) {
        MapResponse response = new MapResponse();
        if (param.getId() == null) {
            response.setFail("id is required");
            return response;
        }
        if (param.getState() == null) {
            response.setFail("state is required");
            return response;
        }
        List<FuncidDefChangeRecord> data = repository.findAllById(Arrays.asList(param.getId()));
        if (data.size() == 0) {
            response.setFail("id not exists");
            return response;
        }
        FuncidDefChangeRecord changeRecord = data.get(0);
        if (param.getState() == 2) {
            changeRecord.setConfirmUser(UserUtils.currentUserId() + "");
            changeRecord.setConfirmTime(new Date());
        }
        if (param.getState() == 3) {
            changeRecord.setPublishUser(UserUtils.currentUserId() + "");
            changeRecord.setPublishTime(new Date());
        }
        if (param.getState() == 4) {
            changeRecord.setFinishTime(new Date());
        }
        repository.save(changeRecord);
        response.add("id", param.getId());
        return response;
    }

    @Transactional
    public MapResponse delete(List<Integer> param) {
        MapResponse response = new MapResponse();
        List<FuncidDefChangeRecord> list = repository.findAllById(param);
        repository.deleteAll(list);
        return response;
    }

    @Transactional
    public void download(HttpServletResponse resp, List<Integer> idList) {
//        List<ChangeRecord> data = repository.findAllById(idList);
//        data.forEach(item -> {
//            item.setState(3);
//            item.setPublishUser(UserUtils.currentUserId() + "");
//            item.setPublishTime(new Date());
//        });
//        repository.saveAll(data);
//
//        String fileName = DateUtils.format(new Date()) + ".sql";
//        List<String> sqlList = getSqlList(idList);
//
//        OutputStream out = null;
//        try {
//            InputStream stream = getStream(sqlList);
//            resp.reset();// 清空输出流
//            String resultFileName = URLEncoder.encode(fileName, "UTF-8");
//            resp.setCharacterEncoding("UTF-8");
//            resp.setHeader("Content-disposition", "attachment; filename=" + resultFileName);// 设定输出文件头
//            resp.setContentType("application/txt");// 定义输出类型
//            out = resp.getOutputStream();
//            byte[] buff = new byte[4096];
//            int size = 0;
//            while ((size = stream.read(buff)) != -1) {
//                out.write(buff, 0, size);
//            }
//            out.flush();
//        } catch (Exception e) {
//            e.printStackTrace();
//        } finally {
//            try {
//                if (out != null) {
//                    out.close();
//                }
//            } catch (IOException e) {
//                e.printStackTrace();
//            }
//        }
    }

    public InputStream getStream(List<String> list) throws IOException {
        ByteArrayOutputStream baos = new ByteArrayOutputStream();
        DataOutputStream out = new DataOutputStream(baos);
        for (String sql : list) {
            out.writeBytes(sql + "\r\n");
        }
        byte[] bytes = baos.toByteArray();
        ByteArrayInputStream tInputStringStream = new ByteArrayInputStream(bytes);
        baos.close();
        out.close();
        return tInputStringStream;
    }

    private List<String> getSqlList(List<Integer> idList) {
//        List<String> sqlList = new ArrayList<>();
//        List<ChangeRecord> data = repository.findAllById(idList);
//        for (ChangeRecord changeRecord : data) {
//            StringBuilder sql = new StringBuilder();
//            Map<String, Object> map = json2map(JacksonMapper.toSimpleJson(changeRecord.getValue()));
//            switch (changeRecord.getOperationType()) {
//                case create:
//                    sql.append("insert into");
//                    if (changeRecord.getInfoCode() == null) {
//                        sql.append(" RWD_DEF_CLASS");
//                    } else {
//                        sql.append(" RWD_DEF_FUNCID");
//                    }
//                    sql.append(" (");//表名
//                    map.keySet().forEach(item -> {
//                        sql.append(humpToUnderline(item) + ",");
//                    });
//                    sql.deleteCharAt(sql.length() - 1);
//                    sql.append(") values (");
//                    map.values().forEach(item -> {
//                        sql.append("'" + item + "',");
//                    });
//                    sql.deleteCharAt(sql.length() - 1);
//                    sql.append(")");
//                    break;
//                case update:
//                    sql.append("update");
//                    if (changeRecord.getInfoCode() == null) {
//                        sql.append(" RWD_DEF_CLASS");
//                    } else {
//                        sql.append(" RWD_DEF_FUNCID");
//                    }
//                    sql.append(" set");
//                    for (Map.Entry<String, Object> entry : map.entrySet()) {
//                        sql.append(" " + humpToUnderline(entry.getKey()) + " = '" + entry.getValue() + "',");
//                    }
//                    sql.deleteCharAt(sql.length() - 1);
//                    break;
//                case delete:
//                    sql.append("delete from");
//                    if (changeRecord.getInfoCode() == null) {
//                        sql.append(" RWD_DEF_CLASS");
//                    } else {
//                        sql.append(" RWD_DEF_FUNCID");
//                    }
//                    break;
//            }
//            if (!EnumOperationType.create.equals(changeRecord.getOperationType())) {
//                sql.append(" where 1=1");
//                if (changeRecord.getInfoCode() == null) {
//                    sql.append(" and code = '" + changeRecord.getCode() + "'");
//                } else {
//                    sql.append(" and class_code = '" + changeRecord.getCode() + "'");
//                    sql.append(" and code = '" + changeRecord.getInfoCode() + "'");
//                }
//            }
//            sql.append(";");
//            sqlList.add(sql.toString());
//        }
//        return sqlList;
        return null;
    }

    public static void main(String[] args) {
        String uuid = UUID.randomUUID().toString().replace("-", "");
        System.err.println(uuid);
    }

    private static Map<String, Object> json2map(String str_json) {
        Map<String, Object> res = null;
        try {
            Gson gson = new Gson();
            res = gson.fromJson(str_json, new TypeToken<Map<String, Object>>() {
            }.getType());
        } catch (JsonSyntaxException e) {
        }
        return res;
    }

    /***
     * 驼峰命名转为下划线命名
     */
    private String humpToUnderline(String para) {
        StringBuilder sb = new StringBuilder(para);
        int temp = 0;//定位
        if (!para.contains("_")) {
            for (int i = 0; i < para.length(); i++) {
                if (Character.isUpperCase(para.charAt(i))) {
                    sb.insert(i + temp, "_");
                    temp += 1;
                }
            }
        }
        return sb.toString().toUpperCase();
    }

}

